from operator import itemgetter

def unescape_name(name):
    if name[0] != '"' and name[-1] != '"':
        return name
    name = name[1:-1]
    unescaped = ""
    i = 0;
    while i < len(name):
        if name[i] != '\\':
            unescaped = unescaped + name[i]
            i = i + 1
        else:
            unescaped = chr(int(name[i+1:i+1+3], 8))
            i = i + 4
    return unescaped
        

class Manifest:
    def __init__ (self, path = None, file = None):
        self.infos = []
        if path:
            self.read_path (path)
        if file:
            self.read_file (file)

    def get_all_blobs (self):
        sha1s = {}
        for info in self.infos:
            if info.has_key("sha1") and info["sha1"] != "empty":
                sha1s[info["sha1"]] = 1
        return sha1s;

    def get_blob_for_name(self, name):
        for i in self.infos:
            if i["name"] == name:
                if i.has_key("sha1"):
                    return i["sha1"]
                else:
                    return None
        return None

    def find_name_for_blob(self, sha1):
        for i in self.infos:
            if i.has_key("sha1") and i["sha1"] == sha1:
                return i["name"]
        return None

    def blobs_changed_in_update (self, target):
        sha1s = {}
        i = 0; j = 0;
        while True:
            a = None
            if i < len(self.infos):
                a = self.infos[i]
            b = None
            if j < len(target.infos):
                b = target.infos[j]

            if a == None and b == None:
                break

            if a == None or (b and a["name"] > b["name"]):
                j = j + 1
            elif b == None or a["name"] < b["name"]:
                i = i + 1
            else:
                if a.has_key("sha1") and a["sha1"] != "empty" and b.has_key("sha1") and b["sha1"] != "empty" and  a["sha1"] != b["sha1"]:
                    sha1s[(a["sha1"], b["sha1"])] = 1
                i = i + 1
                j = j + 1
                
        return sha1s.keys()

    def required_for_update (self, target):
        sha1s = {}
        i = 0; j = 0;
        while True:
            a = None
            if i < len(self.infos):
                a = self.infos[i]
            b = None
            if j < len(target.infos):
                b = target.infos[j]

            if a == None and b == None:
                break
            
            if a == None or (b and a["name"] > b["name"]):
                if b.has_key("sha1") and b["sha1"] != "empty":
                    sha1s[b["sha1"]] = 1
                j = j + 1
            elif b == None or a["name"] < b["name"]:
                i = i + 1
            else:
                if b.has_key("sha1") and b["sha1"] != "empty" and (not a.has_key("sha1") or (a["sha1"] != b["sha1"])):
                    sha1s[b["sha1"]] = 1
                i = i + 1
                j = j + 1
                
        return sha1s

    def parse_line (self, line):
        info = {}
        items = line.split (" ")

        type = items[0];

        if (type != "blob" and
            type != "dir" and
            type != "char" and
            type != "block" and
            type != "fifo" and
            type != "link" and
            type != "socket"):
            return None
        
        info["type"] = type;
        info["mode"] = int(items[1], 8)
        uidgid = items[2].split(":")
        info["uid"] = int(uidgid[0], 10)
        info["gid"] = int(uidgid[1], 10)

        i = 3
        
        if type == "blob":
            info["sha1"] = items[i]
            i = i + 1
        elif type == "char" or type == "block":
            info["dev"] = int(items[i], 10)
            i = i + 1
        elif type == "link":
            info["symlink"] = unescape_name(items[i])
            i = i + 1

        info["name"] = unescape_name(items[i])
        
        return info

    def read_path (self, path):
        f = open (path,'r')
        self.read_file (f)
        f.close()

    def read_file (self, f):
        for line in f.readlines():
            if line[-1:] == '\n':
                line = line[:-1]
            info = self.parse_line (line)
            if info:
                self.infos.append (info)
        self.infos.sort (key=itemgetter('name'))

if __name__ == "__main__":
    import sys
    m1 = Manifest(sys.argv[1])
    if len(sys.argv) == 2:
        for i in m1.get_all_blobs():
            print i
    else:
        m2 = Manifest(sys.argv[2])
        for i in m1.required_for_update(m2):
            print i
        
        
