import os, sys
import urllib2
import gzip
import tempfile

def download_file (in_file, out_file):
    while True:
        data = in_file.read(8192)
        if data == "":
            break
        out_file.write(data)
    out_file.flush()

class Server:
    def __init__(self, base_uri):
        self.protocol = 0
        self.base_uri = base_uri
        self.try_diffs = True
        self.try_gzip = True
        self.failed = False

    def set_protocol(self, protocol):
        self.protocol = protocol

    def set_try_diffs(self, try_diffs):
        self.try_diffs = try_diffs

    def set_try_gzip(self, try_gzip):
        self.try_gzip = try_gzip

    def set_failed(self):
        self.failed = True

    def can_download_from(self):
        if self.failed:
            return False
        return True

    def open_diff(self, from_sha1, to_sha1):
        uri = os.path.join (self.base_uri, "diffs", "%s-%s.diff" %(from_sha1, to_sha1))
        in_file = None
        try:
            in_file = urllib2.urlopen(uri)
        except urllib2.HTTPError:
            pass
        return in_file

    def open_blob(self, sha1):
        if self.try_gzip:
            try:
                uri = os.path.join (self.base_uri, "blobs", sha1 + ".gz")
                zipped_file = urllib2.urlopen(uri)

                # Due to GzipFile requiring seek ops we need to download
                # the file data and use a real file object
                tmp = tempfile.TemporaryFile()
                download_file (zipped_file, tmp)
                tmp.seek(0)
                
                out_file = gzip.GzipFile(sha1, "rb", fileobj=tmp)
                return out_file
            except:
                #print "open_blob gz exception: %s" % (str(sys.exc_info()[1]), )
                pass
            
        uri = os.path.join (self.base_uri, "blobs", sha1)
        return urllib2.urlopen(uri)

    def open_latest_release(self, image_id):
        filename =  "%s.latest" % (image_id,)
    
        uri = os.path.join (self.base_uri, "releases", filename)
        return urllib2.urlopen(uri)
    
    def open_release(self, image_id, version):
        filename =  "%s.%d" % (image_id, version)
    
        uri = os.path.join (self.base_uri, "releases", filename)
        return urllib2.urlopen(uri)

    def __str__(self):
        return "Server{uri:%s,proto:%d}"%(self.base_uri, self.protocol)
