export PATH=$PATH:tools/

if [ ! -f store/image_id ]; then
    echo "You must select what image you are tracking."
    echo "These are common ids:"
    echo "1. 'olpc-devel-ext3'"
    echo "   developer version of olpc image, used on ext3 (e.g. on emulated systems)"
    echo "2. 'olpc-devel-jffs2'"
    echo "   developer version of olpc image, used on jffs2 (i.e. on the XO laptop)"
    echo "Enter the image id name or a number from above"
    IMAGE=""
    while [ "x$IMAGE" == "x" ]; do
	read -e -p "Image: " IMAGE
	if [ "x$IMAGE" == "x1" ]; then
	    IMAGE=olpc-devel-ext3
	elif [ "x$IMAGE" == "x2" ]; then
	    IMAGE=olpc-devel-jffs2
	fi
	if [ "x$IMAGE" != "x" ]; then
	    echo -n "You selected image '$IMAGE', is this right? [yn] "
	    read YESNO
	    if [ "x$YESNO" != "xy" ]; then IMAGE=""; fi
	fi
    done
    echo $IMAGE > store/image_id
fi

IMAGE=`cat store/image_id`

if [ ! -f store/current.mft ]; then
    echo "The manifest for the currently installed image is not known."
    echo "In order to upgrade we need to know the version of '$IMAGE' is installed."
    echo "Please enter the version you have currently installed."
    VERSION=""
    while [ "x$VERSION" == "x" ]; do
	read -e -p "Version: " VERSION
	if [ "x$VERSION" != "x" ]; then
	    echo -n "You entered '$VERSION', is this right? [yn] "
	    read YESNO
	    if [ "x$YESNO" != "xy" ]; then VERSION=""; fi
	fi
    done
    if ./updatinator.py --cat-manifest $IMAGE $VERSION > store/tmp.mft; then true; else
	echo "Unable to download version $VERSION of $IMAGE"
	exit 1
    fi
    echo $VERSION > store/current_version
    mv store/tmp.mft store/current.mft
fi

CURRENT_VERSION=`cat store/current_version`

echo "You're currently on version $CURRENT_VERSION of $IMAGE. "
echo "What version do you want to upgrade to?"
read -e -p "Version: " VERSION

if ./updatinator.py -m store/current.mft -i / --download $IMAGE $VERSION; then true; else
    echo "Failed to download all data"
    exit 1
fi

if ./updatinator.py --cat-manifest $IMAGE $VERSION > store/update.mft; then true; else
    echo "Failed to get target manifest"
    exit 1
fi

echo "All data for upgrade downloaded. Press enter to start upgrade."
read

if upgrade-manifest -b store/blobs/ -d / --delete store/current.mft store/update.mft; then true; else
    echo "Fatal error during upgrade..."
fi
mv store/update.mft store/current.mft
echo $VERSION > store/current_version

echo "Successfully update to $IMAGE $VERSION"
